function grid = domain_to_grid(domain, h)

%
% Step 1: Determine the grid
%
max_grid = max(max(abs(domain)))+2*h;

base_grid = -max_grid:h:max_grid;
len_bg = length(base_grid);

% grid points with id <= -1 have a neighbour in the grid; -id given
% grid points with id ==  0 are not part of the grid
% grid points with id >=  1 are part of the grid; id given
grid_ids = zeros(len_bg);
X = zeros(len_bg);
Y = zeros(len_bg);

for i = 1:len_bg
   x2 = base_grid(i);
   for j = 1:len_bg
      x1 = base_grid(j);

      X(i,j) = x1;
      Y(i,j) = x2;
   end
end

locx = reshape(X, len_bg^2, 1);
locy = reshape(Y, len_bg^2, 1);

cn = inpoly([locx, locy], domain');

inmat = reshape(cn, len_bg, len_bg);

act_id = 0;
for i = 1:len_bg
   for j = 1:len_bg
      if( inmat(i,j) )
         act_id = act_id + 1;
         grid_ids(i,j) = act_id;
      end
   end
end

np = act_id;

grid.x = zeros(2,np);
grid.TRBL = zeros(4,np);
grid.np = np;

grid.grid_ids = grid_ids;
grid.grid_X = X;
grid.grid_Y = Y;

grid.h = h;

act_border_id = 0;

for i = 1:len_bg
   for j = 1:len_bg
      id = grid_ids(i,j);
      if( id>0 )
         grid.x(:,id) = [ base_grid(j); base_grid(i) ];

         [stencil] = get_neighbours(grid_ids, i, j);

         grid.TRBL(:,id) = stencil;
      end
   end
end

grid.h = h;

function [stencil] = get_neighbours(A, i, j)

T = A(i+1, j);
R = A(i, j+1);
B = A(i-1, j);
L = A(i, j-1);

stencil = [T,R,B,L];

function [bpos, normal] = determine_border_pos(h, domain, xval, direction)

if( direction == 1 )
   yval = xval + [ 0;  h ];
elseif( direction == 2 )
   yval = xval + [ h;  0 ];
elseif( direction == 3 )
   yval = xval + [ 0; -h ];
elseif( direction == 4 )
   yval = xval + [-h;  0 ];
end

for i=1:size(domain, 2)-1
   % Compute crossing point
   from = domain(:,i);
   to   = domain(:,i+1);

   p_dir = (to-from); p_dir = p_dir/norm(p_dir);
   g_dir = (xval-yval); g_dir = g_dir/norm(g_dir);
end
